<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 04/20/2004 *
 * ===================                          Last change: 06/20/2010 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : daily_daily.php                                  *
 * -------------------------------------------------------------------- *
 * Short description : Things to do on daily reset                      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Dinge, die beim taeglichen Reset erledigt werden *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} elseif ((!isHtmlOutputMode()) || (!isDailyResetEnabled())) {
	// Do not execute when script is in non-HTML mode or no daily reset
	return;
} elseif ((!isExtensionActive('sql_patches')) || (!isExtensionActive('user'))) {
	if (isDebugModeEnabled()) logDebugMessage(__FILE__, __LINE__, 'Not resetting, needed extension(s) ext-sql_patches/ext-user disabled.');
	return;
}

// Debug line
//* DEBUG: */ logDebugMessage(__FILE__, __LINE__, 'Daily reset started.');

// Update user profiles
if (isExtensionInstalledAndNewer('order', '0.1.1')) {
	// Latest version
	sqlQuery('UPDATE
	`{?_MYSQL_PREFIX?}_user_data`
SET
	`receive_mails`=`max_mails`,
	`mail_orders`=0
WHERE
	`receive_mails` != `max_mails`
	' . runFilterChain('user_exclusion_sql', ' ') . '', __FILE__, __LINE__);
} else {
	// Obsolete version
	sqlQuery('UPDATE
	`{?_MYSQL_PREFIX?}_user_data`
SET
	`receive_mails`=`max_mails`
WHERE
	`receive_mails` != `max_mails`
	' . runFilterChain('user_exclusion_sql', ' ') . '', __FILE__, __LINE__);
}

// Transfer points from locked_points to points
$result_daily = sqlQuery("SELECT
	`userid`
FROM
	`{?_MYSQL_PREFIX?}_user_data`
WHERE
	`ref_payout`=0 AND
	`status`='CONFIRMED'
	" . runFilterChain('user_exclusion_sql', ' ') . "
ORDER BY
	`userid` ASC", __FILE__, __LINE__);

//* DEBUG: */ debugOutput(basename(__FILE__) . ':payout=0;daily|numRows=' . sqlNumRows($result_daily));
if (!ifSqlHasZeroNumRows($result_daily)) {
	// Init SQLs
	initSqls();

	// Run through all 'locked_(foo)_points columsn
	foreach (runFilterChain('locked_points_columns_array', array('points')) as $columnName) {
		// Start checking accounts which are on 0 confirmed-to-go mails
		while ($content = sqlFetchArray($result_daily)) {
			//* DEBUG: */ debugOutput(basename(__FILE__) . ':' . $content['userid']);
			$result_points = sqlQueryEscaped("SELECT
	`ref_depth`,
	`locked_%s`
FROM
	`{?_MYSQL_PREFIX?}_user_points`
WHERE
	`userid`=%s AND
	`locked_%s` > 0.00000
ORDER BY
	`ref_depth` ASC",
				array(
					$columnName,
					bigintval($content['userid']),
					$columnName
				), __FILE__, __LINE__);

			//* DEBUG: */ debugOutput(basename(__FILE__) . ':payout=0;points|numRows=' . sqlNumRows($result_points));
			if (!ifSqlHasZeroNumRows($result_points)) {
				// Ok transfer points
				while ($content2 = sqlFetchArray($result_points)) {
					// Merge both arrays
					$content = merge_array($content, $content2);

					//* DEBUG: */ debugOutput(basename(__FILE__) . ':userid=' . $content['userid'].',depth='.$content['ref_depth'].',columnName=' . $columnName . ',locked='.$content['locked_' . $columnName]);
					if ($content['ref_depth'] > 0) {
						// Level 1+
						addSql(sqlQueryEscaped("UPDATE
	`{?_MYSQL_PREFIX?}_user_points`
SET
	`%s`=`%s`+%s,
	`locked_%s`=0.00000
WHERE
	`userid`=%s AND
	`ref_depth`=%s
LIMIT 1",
							array(
								$columnName,
								$columnName,
								$content['locked_' . $columnName],
								$columnName,
								bigintval($content['userid']),
							$content['ref_depth']
							), __FILE__, __LINE__, FALSE)
						);
					} else {
						// Level zero
						addSql(sqlQueryEscaped("UPDATE
	`{?_MYSQL_PREFIX?}_user_points`
SET
	`%s`=`%s`+%s,
	`locked_%s`=0.00000
WHERE
	`userid`=%s AND
	`ref_depth` IS NULL
LIMIT 1",
							array(
								$columnName,
								$columnName,
								$content['locked_' . $columnName],
								$columnName,
								bigintval($content['userid'])
							), __FILE__, __LINE__, FALSE)
						);
					}

					// Update mediadata as well
					if (isExtensionInstalledAndNewer('mediadata', '0.0.4')) {
						// Update database
						updateMediadataEntry(array('total_points'), 'add', $content['locked_' . $columnName]);
					} // END - if
				} // END - while
			} // END - if

			// Free memory
			sqlFreeResult($result_points);
		} // END - while
	} // END - foreach

	// Run all SQLs
	runFilterChain('run_sqls');
} // END - if

// Free memory
sqlFreeResult($result_daily);

// Debug line
//* DEBUG: */ logDebugMessage(__FILE__, __LINE__, 'Daily reset ended.');

// [EOF]
?>
