<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 08/29/2004 *
 * ===================                          Last change: 08/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : lib-mysql.php                                    *
 * -------------------------------------------------------------------- *
 * Short description : Database layer for PHP 5.4 and older             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Datenbankschicht fuer PHP 5.4 und aelter         *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// SQL queries
function sqlQuery ($sqlString, $file, $line, $enableCodes = TRUE) {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__][$sqlString])) {
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called: ' . $sqlString);

		// Trim SQL string
		$sqlStringModified = trim($sqlString);

		// Empty query string or link is not up?
		if (empty($sqlStringModified)) {
			// Empty SQL string!
			reportBug(__FUNCTION__, __LINE__, sprintf('SQL string is empty, please fix this: file=%s, line=%s',
				basename($file),
				$line
			));
		} elseif (!isSqlLinkUp()) {
			// We should not quietly ignore this
			reportBug(__FUNCTION__, __LINE__, sprintf('Cannot query database: sqlString=%s,file=%s,line=%s',
				$sqlStringModified,
				basename($file),
				$line
			));
		}

		// Remove \t, \n and \r from queries they may confuse some MySQL versions
		$sqlStringModified = str_replace(array(chr(9), PHP_EOL, chr(13)), array(' ', ' ', ' '), $sqlStringModified);

		// Compile config entries out
		$sqlStringModified = sqlPrepareQueryString($sqlStringModified, $enableCodes);

		// Cache it and remember as last SQL query
		$GLOBALS[__FUNCTION__][$sqlString] = $sqlStringModified;
		$GLOBALS['last_sql']               = $sqlStringModified;
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Stored cache: ' . $sqlStringModified);
	} elseif (!isSqlLinkUp()) {
		// Link went down while using cached SQL
		reportBug(__FUNCTION__, __LINE__, 'Link went down while using cached SQL: sqlString=' . $sqlString . ',file=' . basename($file) . ',line=' . $line . ',enableCodes=' . intval($enableCodes));
	} else {
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Cache used: ' . $sqlString);

		// Use cache (to save a lot function calls
		$GLOBALS['last_sql'] = $GLOBALS[__FUNCTION__][$sqlString];

		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Cache is: ' . $sqlString);
	}

	// Starting time
	$querytimeBefore = microtime(TRUE);

	// Run SQL command
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'file=' . basename($file) . ',line=' . $line . ',sql=' . $GLOBALS['last_sql']);
	$result = mysql_query($GLOBALS['last_sql'], getSqlLink())
		or logSqlError($file, $line, 'file='. basename($file) . ',line=' . $line . ':mysql_error()=' . mysql_error() . ',last_query=' . $GLOBALS['last_sql']);
	//* DEBUG: */ logDebugMessage($file, $line, 'sql=' . $GLOBALS['last_sql'] . ',affected=' . sqlAffectedRows() . ',numRows='.(isValidSqlResult($result) ? sqlNumRows($result) : gettype($result)));

	// Calculate query time
	$queryTime = microtime(TRUE) - $querytimeBefore;

	// Add this query to array including timing
	addSqlToDebug($result, $GLOBALS['last_sql'], $queryTime, $file, $line);

	// Save last successfull query
	setConfigEntry('db_last_query', $GLOBALS['last_sql']);

	// Count all query times
	incrementConfigEntry('sql_time', $queryTime);

	// Count this query
	incrementConfigEntry('sql_count');

	// Debug output
	if (isSqlDebugEnabled()) {
		// Is this the first call?
		if (!isset($GLOBALS['sql_first_entry'])) {
			// Write first entry
			appendLineToFile(getCachePath() . 'mysql.log', 'Module=' . getModule());
			$GLOBALS['sql_first_entry'] = TRUE;
		} // END - if

		// Append debug line
		appendLineToFile(getCachePath() . 'mysql.log', basename($file) . '|LINE=' . $line . '|NUM=' . (isValidSqlResult($result) ? sqlNumRows($result) : 'false') . '|AFFECTED=' . sqlAffectedRows() . '|QUERYTIME:' . ($queryTime * 1000) . 'ms): ' . str_replace(array(chr(13), PHP_EOL), array('', ' '), $GLOBALS['last_sql']));
	} // END - if

	// Increment stats entry
	incrementStatsEntry('db_hits');

	// Return the result
	return $result;
}

// SQL num rows
function sqlNumRows ($resource) {
	// Valid link resource?
	if (!isSqlLinkUp()) return FALSE;

	// Link is not up, no rows by default
	$lines = FALSE;

	// Is the result a valid resource?
	if (isset($GLOBALS['sql_numrows'][intval($resource)])) {
		// Use cache
		$lines = $GLOBALS['sql_numrows'][intval($resource)];
	} elseif (isValidSqlResult($resource)) {
		// Get the count of rows from database
		$lines = mysql_num_rows($resource);

		// Remember it in cache
		$GLOBALS['sql_numrows'][intval($resource)] = $lines;
	} else {
		// No resource given, please fix this
		reportBug(__FUNCTION__, __LINE__, 'No resource given! result[]=' . gettype($resource) . ',last_sql=' .  $GLOBALS['last_sql']);
	}

	// Return lines
	return $lines;
}

// SQL affected rows
function sqlAffectedRows () {
	// Valid link resource?
	if (!isSqlLinkUp()) return FALSE;

	// Get affected rows
	$lines = mysql_affected_rows(getSqlLink());

	// Return it
	return $lines;
}

// SQL fetch row
function sqlFetchRow ($resource) {
	// Is $resource valid?
	if ((!isValidSqlResult($resource)) || (!isSqlLinkUp())) return FALSE;

	// Fetch the data and return it
	return mysql_fetch_row($resource);
}

// SQL fetch array
function sqlFetchArray ($resource) {
	// Is $resource valid?
	if ((!isValidSqlResult($resource)) || (!isSqlLinkUp())) return FALSE;

	// Load row as array from database
	$row = mysql_fetch_assoc($resource);

	// Return only arrays here
	if (is_array($row)) {
		// Return row
		return $row;
	} else {
		// Return a false, else some loops would go endless...
		return FALSE;
	}
}

// SQL result
function sqlResult ($resource, $row, $field = '0') {
	// Is $resource valid?
	if ((!isValidSqlResult($resource)) || (!isSqlLinkUp())) return FALSE;

	// Run the result command
	$result = mysql_result($resource, $row, $field);

	// ... and return the result
	return $result;
}

// SQL connect
function sqlConnectToDatabase ($host, $login, $password, $file, $line) {
	// Try to connect
	$linkResource = mysql_connect($host, $login, $password) or logSqlError($file, $line,  mysql_error());

	// Set the link resource
	if (isValidSqlLink($linkResource)) {
		/*
		 * A non-resource (boolean) may happen on installation phase which
		 * shall not be set here. Only valid link resources shall be set so
		 * isSqlLinkUp() will only return 'true' if there is really a
		 * working database link.
		 */
		setSqlLink($file . ':' . __FUNCTION__, $line . ':' . __LINE__, $linkResource);

		// Init charsets (UTF-8 is default now)
		sqlQuery("SET
	`character_set_results`='utf8',
	`character_set_client`='utf8',
	`character_set_connection`='utf8',
	`character_set_database`='utf8',
	`character_set_server`='utf8'", $file . ':' . __FUNCTION__, $line . ':' . __LINE__);

		// Disallow subtraction for unsigned columns
		sqlQuery("SET `sql_mode`='NO_UNSIGNED_SUBTRACTION'", $file . ':' . __FUNCTION__, $line . ':' . __LINE__);
	} // END - if

	// Return the resource
	//* DEBUG: */ logDebugMessage($file . ':' . __FUNCTION__, $line . ':' . __LINE__, 'linkResource[]=' . gettype($linkResource));
	return $linkResource;
}

// SQL select database
function sqlSelectDatabase ($dbName, $file, $line) {
	// Is there still a valid link? If not, skip it.
	if (!isSqlLinkUp()) return FALSE;

	// Return the result
	//* DEBUG: */ logDebugMessage($file . ':' . __FUNCTION__, $line . ':' . __LINE__, 'Selecting database ' . $dbName);
	return mysql_select_db($dbName, getSqlLink()) or logSqlError($file, $line,  mysql_error());
}

// SQL close link
function sqlCloseLink ($file, $line) {
	// Is the link up?
	if (!isSqlLinkUp()) {
		// Skip double close
		//* DEBUG: */ logDebugMessage($file . ':' . __FUNCTION__, $line . ':' . __LINE__, 'Called but no link is open.');
		return FALSE;
	} // END - if

	// Close database link and forget the link
	$close = mysql_close(getSqlLink()) or logSqlError($file . ':' . __FUNCTION__, $line . ':' . __LINE__, mysql_error());

	// Close link in this layer
	unsetSqlLinkUp(__FUNCTION__, __LINE__);

	// Return the result
	//* DEBUG: */ logDebugMessage($file . ':' . __FUNCTION__, $line . ':' . __LINE__, 'close[' . gettype($close) . ']=' . intval($close));
	return $close;
}

// SQL free result
function sqlFreeResult ($resource) {
	if ((!isValidSqlResult($resource)) || (!isSqlLinkUp())) {
		// Abort here
		return FALSE;
	} // END - if

	// Free result
	$res = mysql_free_result($resource);

	// And return that result of freeing it...
	return $res;
}

// Get id from last INSERT command and secure id
function getSqlInsertId () {
	if (!isSqlLinkUp()) return FALSE;
	return bigintval(mysql_insert_id());
}

// Escape a string for the database
function sqlEscapeString ($str, $secureString = TRUE, $strip = TRUE) {
	// Is there cache?
	if (!isset($GLOBALS['sql_escapes']['' . $str . ''])) {
		// Debug message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ' - BEFORE!');

		// Prepare the string here
		$str = sqlPrepareQueryString($str);

		// Debug message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ' - AFTER!');

		// Secure string first? (which is the default behaviour!)
		if ($secureString === TRUE) {
			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',strip=' . intval($strip) . ' - BEFORE!');

			// Then do it here
			$str = secureString($str, $strip);

			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',strip=' . intval($strip) . ' - AFTER!');
		} // END - if

		// Init (invalid) value
		$ret = '!INVALID!';

		if (!isSqlLinkUp()) {
			// Fall-back to escapeQuotes() when there is no link
			$ret = escapeQuotes($str);
		} elseif (function_exists('mysql_real_escape_string')) {
			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str);

			// The new and improved version
			$ret = mysql_real_escape_string($str, getSqlLink());

			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',ret=' . $ret);
		} elseif (function_exists('mysql_escape_string')) {
			// The obsolete function
			$ret = mysql_escape_string($str, getSqlLink());
		} else {
			// If nothing else works, fall back to escapeQuotes() again
			$ret = escapeQuotes($str);
		}

		// Log message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',ret=' . $ret);

		// Cache result
		$GLOBALS['sql_escapes']['' . $str . ''] = $ret;
	} // END - if

	// Log message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',sql_escapes=' . $GLOBALS['sql_escapes']['' . $str . '']);

	// Return it
	return $GLOBALS['sql_escapes']['' . $str . ''];
}

// Set SQL error in global array
function setSqlError ($file, $line, $message) {
	// Remember plain error in last_sql_error
	$GLOBALS['last_sql_error'] = mysql_error();
}

// Checks whether given link is a valid SQL link
function isValidSqlLink ($linkResource) {
	// Is it a resource?
	return is_resource($linkResource);
}

// Checks whether given result is a valid SQL result
function isValidSqlResult ($resultResource) {
	// TODO: Can't this be made better?
	return is_resource($resultResource);
}

// [EOF]
?>
