<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 06/22/2013 *
 * ===================                          Last change: 06/22/2013 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : hourly_transaction.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Hourly reset for transaction hash                *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Stuendlicher Reset fuer Transaktionshashes       *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} elseif ((!isHtmlOutputMode()) || (isHourlyResetEnabled()) || (!isExtensionInstalledAndNewer('sql_patches', '0.7.5'))) {
	// Do not execute when script is in non-HTML mode or no hourly reset
	return;
} elseif (!isExtensionActive('transaction')) {
	if (isDebugModeEnabled()) logDebugMessage(__FILE__, __LINE__, 'Not resetting, needed extension ext-transaction disabled.');
	return;
}

// Debug line
//* DEBUG: */ logDebugMessage(__FILE__, __LINE__, 'hourly reset started.');

sqlQuery("DELETE LOW_PRIORITY FROM
	`{?_MYSQL_PREFIX?}_transaction_log`
WHERE
	(
		`transaction_confirmed`='Y' AND
		(UNIX_TIMESTAMP() - UNIX_TIMESTAMP(`transaction_timestamp`)) >= {?transaction_confirmed_purge_interval?}
	) OR (
		`transaction_fee_collected`='Y' AND
		(UNIX_TIMESTAMP() - UNIX_TIMESTAMP(`transaction_timestamp`)) >= {?transaction_confirmed_purge_interval?}
	)", __FILE__, __LINE__
);

// Look for for "unhashed" transactions
$result = sqlQuery("SELECT
	`transaction_id`,
	`transaction_sender`,
	`transaction_receiver`,
	`transaction_fee_sender`,
	`transaction_fee_receiver`,
	`transaction_original`,
	`transaction_subject`,
	`transaction_timestamp`
FROM
	`{?_MYSQL_PREFIX?}_transaction_log`
WHERE
	`transaction_hash`='INVALID' AND
	`transaction_level` IS NULL
ORDER BY
	`transaction_id` ASC", __FILE__, __LINE__);

// Are there some entries?
if (sqlNumRows($result) > 0) {
	// Load all
	while ($row = sqlFetchArray($result)) {
		// Generate hash from transaction data
		$hash = generateHashFromTransactionData($row);

		// Save hash in transaction
		sqlQueryEscaped("UPDATE
	`{?_MYSQL_PREFIX?}_transaction_log`
SET
	`transaction_hash`='%s',
	`transaction_captcha`=%s
WHERE
	`transaction_id`=%s
LIMIT 1",
			array(
				$hash,
				bigintval($row['transaction_captcha']),
				bigintval($row['transaction_id'])
			), __FILE__, __LINE__
		);
	} // END - while
} // END - if

// Free result
sqlFreeResult($result);

// Debug line
//* DEBUG: */ logDebugMessage(__FILE__, __LINE__, 'hourly reset ended.');

// [EOF]
?>
