<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 02/28/2009 *
 * ===================                          Last change: 02/28/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : session-functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Session-relevant functions                       *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Sitzungsrelevante Funktionen                     *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Unset/set session variables
function setSession ($var, $value) {
	// Abort in CSS mode here
	if (isCssOutputMode()) {
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Is CSS mode:' . $var . '=' . $value);
		return TRUE;
	} // END - if

	// Trim value and session variable
	$var   = trim(secureString($var));

	// Is the value no array?
	if (!is_array($value)) {
		// Then trim it
		$value = trim($value);
	} // END - if

	// Is the session variable set?
	if ((!is_array($value)) && ('' . $value . '' == '') && (isSessionVariableSet($var))) {
		// Remove the session
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'UNSET:' . $var . '=' . getSession($var));
		unset($_SESSION[$var]);
		if (isPhpVersionEqualNewer('5.3.0')) {
			// session_unregister() is deprecated as of 5.3.0
			return TRUE;
		} else {
			// PHP version < 5.3.0
			return session_unregister($var);
		}
	} elseif ((is_array($value)) || (('' . $value . '' != '') && (!isSessionVariableSet($var)))) {
		// Set session
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'SET:' . $var . '=' . $value);
		$_SESSION[$var] = $value;
		if (isPhpVersionEqualNewer('5.3.0')) {
			// session_unregister() is deprecated as of 5.3.0
			return TRUE;
		} else {
			// PHP version < 5.3.0
			return session_register($var);
		}
	} elseif (!empty($value)) {
		// Update session
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'UPDATE:' . $var . '=' . $value);
		$_SESSION[$var] = $value;
		return TRUE;
	}

	// Ignored (but valid)
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'IGNORED:' . $var . '=' . $value);
	return TRUE;
}

// Check whether a session variable is set
function isSessionVariableSet ($var) {
	// Warning: DO NOT call logDebugMessage() from here, this will cause an endless loop
	return (isset($_SESSION[$var]));
}

// Returns whether the value of the session variable or NULL if not set
function getSession ($var) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'var=' . $var . ' - CALLED!');
	// Default is not found ;-)
	$value = NULL;

	// Is the variable there?
	if (isSessionVariableSet($var)) {
		// Then  get it secured!
		if ((isInstaller()) || (!isSqlLinkUp())) {
			// Secure string without escaping (and compiling)
			$value = secureString($_SESSION[$var]);
		} else {
			// Escape string with SQL driver
			$value = sqlEscapeString($_SESSION[$var]);
		}
	} // END - if

	// Return the value
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, $var . '=' . $value . ' - EXIT!');
	return $value;
}

// Get whole session array
function getSessionArray () {
	// Simply return it
	return $_SESSION;
}

// Destroy user session
function destroyMemberSession ($destroy = FALSE) {
	// Reset userid
	initMemberId();

	// Remove all user data from session
	if ($destroy === TRUE) {
		// Destroy whole session
		return destroySession();
	} else {
		return ((setSession('userid', '')) && (setSession('u_hash', '')));
	}
}

// Destroys the admin session
function destroyAdminSession ($destroy = FALSE) {
	// Kill maybe existing session variables including array elements
	setAdminId(0);
	setAdminMd5('');
	setAdminLast(0);

	// Remove "cache"
	unset($GLOBALS['isAdmin']);

	// Destroy session if requested and return status
	if ($destroy === TRUE) {
		return destroySession();
	} // END - if

	// All fine if the session shall not really be destroyed
	return TRUE;
}

// Destroys session and resets some "caches"
function destroySession () {
	// Unset "cache"
	unset($GLOBALS['isValidSession']);

	// Destroy session
	return session_destroy();
}

// Checks whether the session is valid
function isValidSession () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Then determine it
		$GLOBALS[__FUNCTION__] = ((isset($GLOBALS['valid_session'])) && ($GLOBALS['valid_session'] === TRUE) && (isset($_COOKIE[session_name()])));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether all given session data is set
function isSessionDataSet ($sessionData) {
	// Default is set
	$isset = TRUE;

	// Check all
	foreach ($sessionData as $key) {
		// Is this element set?
		$isset = (($isset) && (isSessionVariableSet($key)));
	} // END - foreach

	// Return result
	return $isset;
}

// Initializes session
function initSession () {
	//* NOISY-DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'CALLED!');

	// Is ext-sql_patches there and newer?
	if (isExtensionInstalledAndNewer('sql_patches', '0.5.3')) {
		// Set session save path if set
		if ((isConfigEntrySet('session_save_path')) && (getConfig('session_save_path') != '')) {
			// Please make sure this valid!
			session_save_path(getConfig('session_save_path'));
		} // END - if
	} // END - if

	// Is a session id there?
	if (!isValidSession()) {
		// Start the session
		//* NOISY-DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Initializing session ...');
		$GLOBALS['valid_session']  = session_start();
		$GLOBALS['isValidSession'] = TRUE;

		//* NOISY-DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'session_id=' . session_id());
	} // END - if

	//* NOISY-DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'EXIT!');
}

// [EOF]
?>
